/*
 * Copyright 2015 Max Schuster.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package eu.maxschuster.vaadin.autocompletetextfield;

import java.io.Serializable;
import java.util.Collection;

/**
 * Functional interface that finds {@link AutocompleteSuggestion}s by a given
 * {@link AutocompleteQuery}.
 *
 * @author Max Schuster
 * @see AutocompleteSuggestion
 * @see AutocompleteQuery
 * @see AutocompleteTextFieldExtension
 */
public interface AutocompleteSuggestionProvider extends Serializable {

    /**
     * Finds {@link AutocompleteSuggestion}s for the given
     * {@link AutocompleteQuery}.
     * <p>
     * The implementation should check if
     * {@link AutocompleteQuery#hasLimit()} is {@code true} and limit its
     * suggestions according to the limit provided by
     * {@link AutocompleteQuery#getLimit()}<br>
     * <strong>Excess suggestions will be ignored by the
     * {@link AutocompleteTextFieldExtension}!</strong>
     * </p>
     *
     * @param query The {@link AutocompleteQuery} generated by a
     * {@link AutocompleteTextFieldExtension}.
     * @return A {@link Collection} of {@link AutocompleteSuggestion}s for the
     * given {@link AutocompleteQuery}
     * @see AutocompleteQuery#hasLimit()
     * @see AutocompleteQuery#getLimit()
     */
    public Collection<AutocompleteSuggestion> querySuggestions(AutocompleteQuery query);

}
